C SUBROUTINE PACKAGE 'RAYCB' FOR COMPLETE RAY TRACING WITHIN ONE COMPLEX
C BLOCK
C
C BY VLASTISLAV CERVENY, LUDEK KLIMES, IVAN PSENCIK
C
C THIS PACKAGE CONSISTS OF THE FOLLOWING SUBROUTINES:
C     RAYCB...SUBROUTINE TRANSFERING THE QUANTITIES GIVEN AT THE INITIAL
C             POINT OF THE ELEMENT OF A RAY INTO THE CORRESPONDING
C             QUANTITIES AT THE ENDPOINT OF THE ELEMENT (SEE
C             C.R.T.5.8.1).
C     FCT...  SUBROUTINE EVALUATING THE RIGHT-HAND SIDES OF THE SYSTEM
C             OF ORDINARY DIFFERENTIAL EQUATIONS FOR Y(1) TO Y(27) (SEE
C             C.R.T.5.8.2).
C     OUTP... SUBROUTINE CONTAINING VARIOUS TESTS OF THE POSITION OF THE
C             NEWLY COMPUTED POINT OF THE RAY, TESTS FOR POSSIBLE
C             CAUSTIC POINTS, ETC. (SEE C.R.T.5.8.3).
C     SMVPRD..AUXILIARY SUBROUTINE TO FCT, EVALUATES SYMMETRIC MATRIX BY
C             VECTOR PRODUCT.
C     PSHIFT..AUXILIARY SUBROUTINE TO OUTP. IT CORRECTS REDUCED
C             AMPLITUDES WITH REGARD TO PHASE SHIFT DUE TO CAUSTICS (SEE
C             C.R.T.5.8.3F).
C     CROSS...SUBROUTINE DESIGNED TO FIND THE POINT OF INTERSECTION OF A
C             CURVE WITH A SURFACE (SEE C.R.T.5.8.4B).
C     HIVD2...SUBROUTINE PERFORMING THE HERMITE INTERPOLATION OF A
C             VECTOR AND ITS DERIVATIVES USING FUNCTIONAL VALUES AND
C             DERIVATIVES AT 2 GIVEN POINTS (SEE C.R.T.5.8.4A).
C
C DATE: 1989, DECEMBER 6
C
C=======================================================================
C
      SUBROUTINE RAYCB(YL,Y,YY,IY)
      REAL YL(6),Y(35),YY(5)
      INTEGER IY(12)
C
C THIS SUBROUTINE TRANSFERS THE QUANTITIES GIVEN AT THE INITIAL POINT OF
C THE ELEMENT OF A RAY INTO THE CORRESPONDING QUANTITIES AT THE ENDPOINT
C OF THE ELEMENT (SEE C.R.T.5.8.1).
C
C INPUT:
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE INITIAL POINT OF
C             THE ELEMENT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY
C             AT THE INITIAL POINT OF THE ELEMENT OF THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY AT THE INITIAL POINT OF THE ELEMENT OF THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY AT THE INITIAL POINT OF THE ELEMENT OF THE
C             RAY.
C
C OUTPUT:
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE ENDPOINT OF THE
C             ELEMENT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY
C             AT THE ENDPOINT OF THE ELEMENT OF THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY AT THE ENDPOINT OF THE ELEMENT OF THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY AT THE ENDPOINT OF THE ELEMENT OF THE RAY.
C
C COMMON BLOCK /DCRT/ (SEE SUBROUTINE PACKAGE 'RAY'):
      INTEGER MEND,MSTOR
      PARAMETER (MEND=128)
      PARAMETER (MSTOR=128)
      INTEGER KSTORE,NEXPS,NHLF,MODCRT
      REAL STORE,STEP,UEB,UEBPP,UEBPH,UEBHH,UEBDRT,BOUNDR(7)
      INTEGER NSRFCA,NEND,KEND(MEND),NSTOR,KSTOR(MSTOR)
      COMMON/DCRT/ KSTORE,NEXPS,NHLF,MODCRT,STORE,STEP,UEB,UEBPP,UEBPH,
     *             UEBHH,UEBDRT,BOUNDR,NSRFCA,NEND,KEND,NSTOR,KSTOR
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C COMMON BLOCK /INITC/ (SEE SUBROUTINE PACKAGE 'INIT') IS REQUIRED IN
C     SUBROUTINE OUTP.  NONE OF THE STORAGE LOCATIONS OF THE COMMON
C     BLOCK ARE ALTERED THERE.
C
C COMMON BLOCK /RAYC/ -  COMMUNICATION BETWEEN RAYCB, FCT, AND OUTP:
C     ------------------------------------------------------------------
      REAL YLRC(6),YYRC(5),DELT11,DELT13,DELT33,VNEXPS
      INTEGER IYRC(12)
      COMMON/RAYC/YLRC,YYRC,IYRC,DELT11,DELT13,DELT33,VNEXPS
C     ------------------------------------------------------------------
C     YLRC,YYRC,IYRC... WORKING COPIES OF THE ARRAYS YL,YY,YI.
C     DELT11,DELT13,DELT33... DEVIATIONS (IN ABSOLUTE VALUES OF THE TWO
C             COMPUTED BASIS VECTORS OF THE RAY-CENTRED COORDINATE
C             SYSTEM FROM THE CONDITIONS OF ORTHONORMALITY, SEE
C             C.R.T.5.8.2D.
C     VNEXPS..VELOCITY**(-NEXPS).
C YLRC,YYRC,IYRC ARE DEFINED IN THIS SUBROUTINE.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL METRIC,HPCG
C     FUNCTION NSRFC()... SUBROUTINE PACKAGE 'MODEL'.
C     SUBROUTINE FCT(X,Y,D)... THIS PACKAGE.
C     SUBROUTINE OUTP(X,Y,D,IHLF,NDIM,PRMT)... THIS PACKAGE.
C     SUBROUTINE SMVPRD(G,A1,A2,A3,B1,B2,B3)... THIS PACKAGE.
C     SUBROUTINE PSHIFT(NY,Y,YI,Q111,Q121,Q112,Q122,Q1DET,Q211,Q221,
C             Q212,Q222,Q2DET,ISHIFT)... THIS PACKAGE.
C     SUBROUTINE CROSS(ISRF,NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,F)... THIS
C             PACKAGE.
C     SUBROUTINE HIVD2(NDIM,X1,Y1,D1,X2,Y2,D2,X,Y,D)... THIS PACKAGE.
C     SUBROUTINE HPCG(PRMT,Y,DERY,NDIM,IHLF,FCT,OUTP,AUX)... IBM
C             SCIENTIFIC SUBROUTINE PACKAGE.
C     SUBROUTINE RPAR31(YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE 'RPAR'.
C     SUBROUTINE RPAR32(ISTOR,YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE
C             'RPAR'.
C     SUBROUTINE WRIT31(YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE 'WRIT'.
C     SUBROUTINE WRIT32(ISTOR,YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE
C             'WRIT'.
C     SUBROUTINE METRIC(COOR,GSQRD,G,GAMMA)
C     SUBROUTINE BLOCK(COOR,ISRF1,ISB1,ISRF2,ISB2,ICB2)... SUBROUTINE
C             PACKAGE 'MODEL'.
C     SUBROUTINE VELOC(IWAVE,UP,US,QP,QS,VP,VS,VD,QL)... SUBROUTINE
C             PACKAGE 'MODEL'.
C     SUBROUTINE SRFC2(ISRF,COOR,F) AND SUBSEQUENT ROUTINES...
C             SUBROUTINE PACKAGE 'SRFC'.
C     SUBROUTINE PARM2(ICB,COOR,UP,US,RO,QP,QS) AND SUBSEQUENT ROUTINES
C             ... SUBROUTINE PACKAGE 'PARM'.
C
C DATE: 1990, JANUARY 25
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS FOR LOCAL MODEL PARAMETERS:
      REAL G(12),GAMMA(18),GSQRD,FAUX(10)
      REAL UP(10),US(10),RO,QP,QS,VP,VS,VD(10),QL
C     THESE AUXILIARY VARIABLES AND ARRAYS NEED NOT BE LOCATED IN A
C     COMMON BLOCK.  THERE IS NO REASON TO LOCATE THEM IN THE AUXILIARY
C     COMMON BLOCK /AUXMOD/ BUT TO SHARE THE MEMORY.
      COMMON/AUXMOD/ G,GAMMA,GSQRD,FAUX,UP,US,RO,QP,QS,VP,VS,VD,QL
C
C     G,GAMMA,GSQRD... SEE SUBROUTINE METRIC OF THE PACKAGE 'METRIC'.
C     FAUX... AUXILIARY ARRAY TO STORE A FUNCTIONAL VALUE AND ITS
C             DERIVATIVES.
C     UP,US,RO,QP,QS... SEE SUBROUTINE PARM2 OF THE PACKAGE 'PARM'.
C     UP,US,QP,QS,VP,VS,VD,QL... SEE SUBROUTINE VELOC OF THE PACKAGE
C             'MODEL'.
C
C.......................................................................
C
C     EXTERNAL PROCEDURE NAMES:
      EXTERNAL FCT,OUTP
C     THESE SUBROUTINES ARE CALLED BY THE SUBROUTINE HPCG.
C
C     AUXILIARY STORAGE LOCATIONS:
      INTEGER NDIM,IHLF,I
      PARAMETER (NDIM=27)
      REAL PRMT(6),DERY(NDIM),AUX(16,NDIM),VRED,AUX1
C
C     NDIM,IHLF,PRMT,DERY,AUX... SEE SUBROUTINE HPCG OF THE IBM
C             SCIENTIFIC SUBROUTINE PACKAGE.
C     I...    AUXILIARY LOOP VARIABLE.
C     VRED... APPROXIMATE REFERENCE VELOCITY TO ESTIMATE ERROR WEIGHTS.
C     AUX1... AUXILIARY STORAGE LOCATION.
C
C.......................................................................
C
C     (A) STORING AUXILIARY INPUT QUANTITIES IN COMMON BLOCK /RAYC/:
      DO 11 I=1,6
        YLRC(I)=YL(I)
   11 CONTINUE
      DO 12 I=1,5
        YYRC(I)=YY(I)
   12 CONTINUE
      DO 13 I=1,12
        IYRC(I)=IY(I)
   13 CONTINUE
C
C     (B) INITIAL VALUES FOR NUMERICAL INTEGRATION:
   20 PRMT(1)=YYRC(1)
      PRMT(2)=PRMT(1)+999999.
      PRMT(3)=STEP
      PRMT(4)=13.444*YYRC(2)
      CALL METRIC(Y(3),GSQRD,G,GAMMA)
      IF(IYRC(5).GE.0) THEN
        VRED=YLRC(1)
      ELSE
        VRED=YLRC(2)
      END IF
      DERY(1)=1.
      DERY(2)=1.
      DERY(3)=SQRT(G(1))/VRED
      DERY(4)=SQRT(G(3))/VRED
      DERY(5)=SQRT(G(6))/VRED
      AUX1=STEP*VRED**(1-NEXPS)
      DERY(6)=SQRT(G(7))*AUX1
      DERY(7)=SQRT(G(9))*AUX1
      DERY(8)=SQRT(G(12))*AUX1
      DO 21 I=9,NDIM
        DERY(I)=0.
   21 CONTINUE
C
C     (C) NUMERICAL INTEGRATION:
      CALL HPCG(PRMT,Y,DERY,NDIM,IHLF,FCT,OUTP,AUX)
C
C     (D) GREAT NUMBER OF BISECTIONS:
      IF(PRMT(5).LT.0.) THEN
        YYRC(2)=2.*YYRC(2)
        GO TO 20
      END IF
C
C     (E) RECALLING AUXILIARY INPUT QUANTITIES FROM COMMON BLOCK /RAYC/:
      DO 51 I=1,6
        YL(I)=YLRC(I)
   51 CONTINUE
      DO 52 I=1,5
        YY(I)=YYRC(I)
   52 CONTINUE
      DO 53 I=1,12
        IY(I)=IYRC(I)
   53 CONTINUE
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE FCT(X,Y,D)
      INTEGER NDIM
      PARAMETER (NDIM=27)
      REAL X,Y(NDIM),D(NDIM)
C
C THIS SUBROUTINE EVALUATES THE RIGHT-HAND SIDES OF THE SYSTEM OF
C ORDINARY DIFFERENTIAL EQUATIONS FOR Y(1) TO Y(27). SEE C.R.T., SECTION
C 5.8.2.
C
C INPUT:
C     X...    VALUE OF THE INDEPENDENT VARIABLE ALONG THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C NONE OF THE INPUT PARAMETERS EXCEPT Y(3)-Y(8) IS ALTERED.
C
C OUTPUT:
C     D...    ARRAY CONTAINING DERIVATIVES OF THE BASIC QUANTITIES
C             COMPUTED ALONG THE RAY, WITH RESPECT TO THE INDEPENDENT
C             VARIABLE ALONG THE RAY.
C     Y(3:8)..RENORMALIZED ORTHOGONAL VECTORS.  THE MODIFICATION SHOULD
C             BE NEGLIGIBLE.
C
C COMMON BLOCK /DCRT/ (SEE SUBROUTINE PACKAGE 'RAY'):
      INTEGER MEND,MSTOR
      PARAMETER (MEND=128)
      PARAMETER (MSTOR=128)
      INTEGER KSTORE,NEXPS,NHLF,MODCRT
      REAL STORE,STEP,UEB,UEBPP,UEBPH,UEBHH,UEBDRT,BOUNDR(7)
      INTEGER NSRFCA,NEND,KEND(MEND),NSTOR,KSTOR(MSTOR)
      COMMON/DCRT/ KSTORE,NEXPS,NHLF,MODCRT,STORE,STEP,UEB,UEBPP,UEBPH,
     *             UEBHH,UEBDRT,BOUNDR,NSRFCA,NEND,KEND,NSTOR,KSTOR
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C COMMON BLOCK /RAYC/ -  COMMUNICATION BETWEEN RAYCB, FCT, AND OUTP:
      REAL YLRC(6),YYRC(5),DELT11,DELT13,DELT33,VNEXPS
      INTEGER IYRC(12)
      COMMON/RAYC/YLRC,YYRC,IYRC,DELT11,DELT13,DELT33,VNEXPS
C YLRC,IYRC ARE MODIFIED IN THIS SUBROUTINE.  DELT11,DELT13,DELT33 AND
C VNEXPS ARE DEFINED IN THIS SUBROUTINE.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL KOOR,METRIC,PARM2,VELOC,SMVPRD
      INTEGER KOOR
C     SUBROUTINE SMVPRD(G,A1,A2,A3,B1,B2,B3)... THIS PACKAGE.
C     SUBROUTINE METRIC(COOR,GSQRD,G,GAMMA)
C     FUNCTION KOOR()... SUBROUTINE PACKAGE 'METRIC'.
C     SUBROUTINE VELOC(IWAVE,UP,US,QP,QS,VP,VS,VD,QL)... SUBROUTINE
C             PACKAGE 'MODEL'.
C     SUBROUTINE PARM2(ICB,COOR,UP,US,RO,QP,QS) AND SUBSEQUENT ROUTINES
C             ... SUBROUTINE PACKAGE 'PARM'.
C
C DATE: 1990, JANUARY 25
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS FOR LOCAL MODEL PARAMETERS:
      REAL G(12),GAMMA(18),GSQRD,FAUX(10)
      REAL UP(10),US(10),RO,QP,QS,VP,VS,VD(10),QL
C     THESE AUXILIARY VARIABLES AND ARRAYS NEED NOT BE LOCATED IN A
C     COMMON BLOCK.  THERE IS NO REASON TO LOCATE THEM IN THE AUXILIARY
C     COMMON BLOCK /AUXMOD/ BUT TO SHARE THE MEMORY.
      COMMON/AUXMOD/ G,GAMMA,GSQRD,FAUX,UP,US,RO,QP,QS,VP,VS,VD,QL
C
C     G,GAMMA,GSQRD... SEE SUBROUTINE METRIC OF THE PACKAGE 'METRIC'.
C     FAUX... AUXILIARY ARRAY TO STORE A FUNCTIONAL VALUE AND ITS
C             DERIVATIVES.
C     UP,US,RO,QP,QS... SEE SUBROUTINE PARM2 OF THE PACKAGE 'PARM'.
C     UP,US,QP,QS,VP,VS,VD,QL... SEE SUBROUTINE VELOC OF THE PACKAGE
C             'MODEL'.
C
C.......................................................................
C
C     AUXILIARY STORAGE LOCATIONS:
C
      REAL H11,H21,H31,H41,H51,H61,H42,H52,H62,H13,H23,H33,H43,H53,H63
      REAL D11,D13,D33,V,V1,V11,V12,V22,AUX1,AUX2,AUX3,AUX4
      REAL AUX11,AUX21,AUX31,AUX12,AUX22,AUX32,AUX13,AUX23,AUX33
C
C     H11,H21,H31,H13,H23,H33... COVARIANT COMPONENTS OF THE BASIS
C             VECTORS OF RAY-CENTRED COORDINATE SYSTEM.
C     H41,H51,H61,H42,H52,H62,H43,H53,H63... CONTRAVARIANT COMPONENTS OF
C             THE BASIS VECTORS OF RAY-CENTRED COORDINATE SYSTEM.  H41,
C             H51,H61,H43,H53,H63 ARE NOT USED IN CARTESIAN COORDINATES.
C     D11,D13,D33... NORMS AND SCALAR PRODUCT OF BASIS VECTORS H1, H2.
C     V,V1,V11,V12,V22,... VELOCITY AND VELOCITY DERIVATIVES IN
C             RAY-CENTRED COORDINATE SYSTEM.
C     AUX1,AUX2,AUX3,AUX4... AUXILIARY STORAGE LOCATIONS.
C     AUX11,AUX21,AUX31,AUX12,AUX22,AUX32,AUX13,AUX23,AUX33... AUXILIARY
C             STORAGE LOCATIONS.  NOT USED IN CARTESIAN COORDINATES.
C
C.......................................................................
C
C     (A) NUMBER OF INVOCATIONS OF FCT:
      IYRC(9)=IYRC(9)+1
C
C     (B) MATERIAL PARAMETERS:
      CALL PARM2(IABS(IYRC(5)),Y(3),UP,US,YLRC(3),QP,QS)
      CALL VELOC(IYRC(5),UP,US,QP,QS,YLRC(1),YLRC(2),VD,QL)
      YLRC(4)=VD(2)
      YLRC(5)=VD(3)
      YLRC(6)=VD(4)
      V=VD(1)
      VNEXPS=V**(-NEXPS)
C
C     (C) BASIS OF THE RAY-CENTRED COORDINATE SYSTEM:
C     (C1) NON-UNIT VECTORS - COVARIANT COMPONENTS
      H13=Y(6)
      H23=Y(7)
      H33=Y(8)
      H11=Y(9)
      H21=Y(10)
      H31=Y(11)
      IF(KOOR().NE.0) THEN
C       CURVILINEAR COORDINATES:
        CALL METRIC(Y(3),GSQRD,G,GAMMA)
C       (C2) NON-UNIT VECTORS - CONTRAVARIANT COMPONENTS
        CALL SMVPRD(G(7),H13,H23,H33,H43,H53,H63)
        CALL SMVPRD(G(7),H11,H21,H31,H41,H51,H61)
C       (C3) NORMS:
        D33=SQRT(H13*H43+H23*H53+H33*H63)
        D11=SQRT(H11*H41+H21*H51+H31*H61)
        D13=(H11*H43+H21*H53+H31*H63)/(D11*D33)
C       (C4) ORTHONORMAL VECTORS:
C       COVARIANT COMPONENTS
        H13=H13/D33
        H23=H23/D33
        H33=H33/D33
        H11=H11/D11-H13*D13
        H21=H21/D11-H23*D13
        H31=H31/D11-H33*D13
C       CONTRAVARIANT COMPONENTS
        H43=H43/D33
        H53=H53/D33
        H63=H63/D33
        H41=H41/D11-H43*D13
        H51=H51/D11-H53*D13
        H61=H61/D11-H63*D13
        H42=(H23*H31-H33*H21)/GSQRD
        H52=(H33*H11-H13*H31)/GSQRD
        H62=(H13*H21-H23*H11)/GSQRD
C
C       (D) QUANTITIES USEFUL TO TEST THE ACCURACY OF COMPUTATIONS:
        DELT11=ABS(D11-1.)
        DELT33=ABS(V*D33-1.)
        DELT11=ABS(D13)
C
C       (E) FIRST AND SECOND DERIVATIVES OF THE VELOCITY IN RAY-CENTRED
C       COORDINATE SYSTEM:
C       SECOND COVARIANT VELOCITY DERIVATIVES:
        VD(5)=VD(5)-GAMMA(1)*VD(2)-GAMMA(7)*VD(3)-GAMMA(13)*VD(4)
        VD(6)=VD(6)-GAMMA(2)*VD(2)-GAMMA(8)*VD(3)-GAMMA(14)*VD(4)
        VD(7)=VD(7)-GAMMA(3)*VD(2)-GAMMA(9)*VD(3)-GAMMA(15)*VD(4)
        VD(8)=VD(8)-GAMMA(4)*VD(2)-GAMMA(10)*VD(3)-GAMMA(16)*VD(4)
        VD(9)=VD(9)-GAMMA(5)*VD(2)-GAMMA(11)*VD(3)-GAMMA(17)*VD(4)
        VD(10)=VD(10)-GAMMA(6)*VD(2)-GAMMA(12)*VD(3)-GAMMA(18)*VD(4)
C       RAY-CENTRED COORDINATE SYSTEM:
        V1=VD(2)*H41+VD(3)*H51+VD(4)*H61
        CALL SMVPRD(VD(5),H41,H51,H61,AUX1,AUX2,AUX3)
        V11=AUX1*H41+AUX2*H51+AUX3*H61
        V12=AUX1*H42+AUX2*H52+AUX3*H62
        CALL SMVPRD(VD(5),H42,H52,H62,AUX1,AUX2,AUX3)
        V22=AUX1*H42+AUX2*H52+AUX3*H62
C
C       (F) CORRECTION OF THE COMPUTED QUANTITIES:
        Y(6)=H13/V
        Y(7)=H23/V
        Y(8)=H33/V
        Y(9)=H11
        Y(10)=H21
        Y(11)=H31
C
C       (G) RIGHT-HAND SIDES OF THE DIFFERENTIAL EQUATIONS:
        AUX1=V*VNEXPS
        AUX2=V*AUX1
        AUX3=VNEXPS/V
        AUX4=V1*VNEXPS
        D(1)=VNEXPS
        D(2)=0.5*QL*VNEXPS
        D(3)=H43*AUX1
        D(4)=H53*AUX1
        D(5)=H63*AUX1
        CALL SMVPRD(GAMMA(1),H43,H53,H63,AUX11,AUX21,AUX31)
        CALL SMVPRD(GAMMA(7),H43,H53,H63,AUX12,AUX22,AUX32)
        CALL SMVPRD(GAMMA(13),H43,H53,H63,AUX13,AUX23,AUX33)
        D(6)=-VD(2)*AUX3+(AUX11*H13+AUX12*H23+AUX13*H33)*VNEXPS
        D(7)=-VD(3)*AUX3+(AUX21*H13+AUX22*H23+AUX23*H33)*VNEXPS
        D(8)=-VD(4)*AUX3+(AUX31*H13+AUX32*H23+AUX33*H33)*VNEXPS
        D(9) =H13*AUX4+(AUX11*H11+AUX12*H21+AUX13*H31)*AUX1
        D(10)=H23*AUX4+(AUX21*H11+AUX22*H21+AUX23*H31)*AUX1
        D(11)=H33*AUX4+(AUX31*H11+AUX32*H21+AUX33*H31)*AUX1
      ELSE
C       CARTESIAN COORDINATES (20 OF THESE STATEMENT LINES ARE THE SAME
C       AS FOR CURVILINEAR COORDINATES):
C       (C2) NON-UNIT VECTORS - CONTRAVARIANT COMPONENTS
C       H43=H13, H53=H23, H63=H33, H41=H11, H51=H21, H61=H31
C       (C3) NORMS:
        D33=SQRT(H13*H13+H23*H23+H33*H33)
        D11=SQRT(H11*H11+H21*H21+H31*H31)
        D13=(H11*H13+H21*H23+H31*H33)/(D11*D33)
C       (C4) ORTHONORMAL VECTORS:
C       COVARIANT=CONTRAVARIANT COMPONENTS
        H13=H13/D33
        H23=H23/D33
        H33=H33/D33
        H11=H11/D11-H13*D13
        H21=H21/D11-H23*D13
        H31=H31/D11-H33*D13
        H12=(H23*H31-H33*H21)
        H22=(H33*H11-H13*H31)
        H32=(H13*H21-H23*H11)
C
C       (D) QUANTITIES USEFUL TO TEST THE ACCURACY OF COMPUTATIONS:
        DELT11=ABS(D11-1.)
        DELT33=ABS(V*D33-1.)
        DELT11=ABS(D13)
C
C       (E) FIRST AND SECOND DERIVATIVES OF THE VELOCITY IN RAY-CENTRED
C       COORDINATE SYSTEM:
        V1=VD(2)*H11+VD(3)*H21+VD(4)*H31
        CALL SMVPRD(VD(5),H11,H21,H31,AUX1,AUX2,AUX3)
        V11=AUX1*H11+AUX2*H21+AUX3*H31
        V12=AUX1*H12+AUX2*H22+AUX3*H32
        CALL SMVPRD(VD(5),H12,H22,H32,AUX1,AUX2,AUX3)
        V22=AUX1*H12+AUX2*H22+AUX3*H32
C
C       (F) CORRECTION OF THE COMPUTED QUANTITIES:
        Y(6)=H13/V
        Y(7)=H23/V
        Y(8)=H33/V
        Y(9)=H11
        Y(10)=H21
        Y(11)=H31
C
C       (G) RIGHT-HAND SIDES OF THE DIFFERENTIAL EQUATIONS:
        AUX1=V*VNEXPS
        AUX2=V*AUX1
        AUX3=VNEXPS/V
        AUX4=V1*VNEXPS
        D(1)=VNEXPS
        D(2)=0.5*QL*VNEXPS
        D(3)=H13*AUX1
        D(4)=H23*AUX1
        D(5)=H33*AUX1
        D(6)=-VD(2)*AUX3
        D(7)=-VD(3)*AUX3
        D(8)=-VD(4)*AUX3
        D(9)=H13*AUX4
        D(10)=H23*AUX4
        D(11)=H33*AUX4
      END IF
      V11=-V11*AUX3
      V12=-V12*AUX3
      V22=-V22*AUX3
      D(12)=AUX2*Y(14)
      D(13)=AUX2*Y(15)
      D(14)=V11*Y(12)+V12*Y(13)
      D(15)=V12*Y(12)+V22*Y(13)
      D(16)=AUX2*Y(18)
      D(17)=AUX2*Y(19)
      D(18)=V11*Y(16)+V12*Y(17)
      D(19)=V12*Y(16)+V22*Y(17)
      D(20)=AUX2*Y(22)
      D(21)=AUX2*Y(23)
      D(22)=V11*Y(20)+V12*Y(21)
      D(23)=V12*Y(20)+V22*Y(21)
      D(24)=AUX2*Y(26)
      D(25)=AUX2*Y(27)
      D(26)=V11*Y(24)+V12*Y(25)
      D(27)=V12*Y(24)+V22*Y(25)
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE OUTP(X,Y,D,IHLF,NDIM,PRMT)
      INTEGER IHLF,NDIM,MY
      PARAMETER (MY=35)
      REAL X,Y(MY),D(MY),PRMT(5)
C
C THIS SUBROUTINE INCLUDES VARIOUS TESTS OF THE POSITION OF THE NEWLY
C COMPUTED POINT OF THE RAY, TESTS FOR POSSIBLE CAUSTIC POINTS, ETC.  IT
C ALSO STORES THE COMPUTED QUANTITIES INTO PROPER FILES IF REQUIRED.
C
C INPUT:
C     X...    VALUE OF THE INDEPENDENT VARIABLE ALONG THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     D...    ARRAY CONTAINING DERIVATIVES OF THE BASIC QUANTITIES
C             COMPUTED ALONG THE RAY, WITH RESPECT TO THE INDEPENDENT
C             VARIABLE ALONG THE RAY.
C     IHLF... NUMBER OF BISECTIONS OF THE INITIAL INCREMENT OF
C             INDEPENDENT VARIABLE.
C     NDIM... NUMBER OF ORDINARY DIFFERENTIAL EQUATIONS.
C     PRMT... ARRAY CONTAINING PARAMETERS OF THE INTEGRATION.
C NONE OF THE INPUT PARAMETERS EXCEPT X,Y,D,PRMT(5) IS ALTERED.
C
C OUTPUT:
C     X,Y,D...VALUES CORRESPONDING TO THE POINT OF INTERSECTION OF THE
C             RAY WITH THE BOUNDARY OF THE COMPLEX BLOCK OR THE
C             COMPUTATIONAL VOLUME IF THE RAY INTERSECTS THE BOUNDARY.
C             UNALTERED IF THE RAY DOES NOT INTERSECT THE BOUNDARY.
C     PRMT(5)=1.0... THE RAY HAS INTERSECTED THE BOUNDARY OF THE COMPLEX
C               BLOCK OR THE COMPUTATIONAL VOLUME,
C             -1.0... NUMBER IHLF OF BISECTIONS OF THE INITIAL INCREMENT
C               GREATER THAN THE SPECIFIED LIMIT NHLF,
C             OTHERWISE UNALTERED.
C
C COMMON BLOCK /DCRT/ (SEE SUBROUTINE PACKAGE 'RAY'):
      INTEGER MEND,MSTOR
      PARAMETER (MEND=128)
      PARAMETER (MSTOR=128)
      INTEGER KSTORE,NEXPS,NHLF,MODCRT
      REAL STORE,STEP,UEB,UEBPP,UEBPH,UEBHH,UEBDRT,BOUNDR(7)
      INTEGER NSRFCA,NEND,KEND(MEND),NSTOR,KSTOR(MSTOR)
      COMMON/DCRT/ KSTORE,NEXPS,NHLF,MODCRT,STORE,STEP,UEB,UEBPP,UEBPH,
     *             UEBHH,UEBDRT,BOUNDR,NSRFCA,NEND,KEND,NSTOR,KSTOR
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C COMMON BLOCK /INITC/ (SEE SUBROUTINE PACKAGE 'INIT'):
      INTEGER MSRFCA
      PARAMETER (MSRFCA=128)
      INTEGER ISB1I,ICB1I
      REAL YLI(6),YI(25),FSRFCA(MSRFCA)
      COMMON/INITC/ISB1I,ICB1I,YLI,YI,FSRFCA
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C COMMON BLOCK /RAYC/ -  COMMUNICATION BETWEEN RAYCB, FCT, AND OUTP:
      REAL YLRC(6),YYRC(5),DELT11,DELT13,DELT33,VNEXPS
      INTEGER IYRC(12)
      COMMON/RAYC/YLRC,YYRC,IYRC,DELT11,DELT13,DELT33,VNEXPS
C YLRC,YYRC,IYRC ARE MODIFIED IN THIS SUBROUTINE.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL SRFC2,PARM2,BLOCK,VELOC,RPAR31,RPAR32,WRIT31,WRIT32
      EXTERNAL NSRFC,CROSS,HIVD2,PSHIFT
      INTEGER NSRFC
C     FUNCTION NSRFC()... SUBROUTINE PACKAGE 'MODEL'.
C     SUBROUTINE PSHIFT(NY,Y,YI,Q111,Q121,Q112,Q122,Q1DET,Q211,Q221,
C             Q212,Q222,Q2DET,ISHIFT)... THIS PACKAGE.
C     SUBROUTINE CROSS(ISRF,NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,F)... THIS
C             PACKAGE.
C     SUBROUTINE HIVD2(NDIM,X1,Y1,D1,X2,Y2,D2,X,Y,D)... THIS PACKAGE.
C     SUBROUTINE RPAR31(YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE 'RPAR'.
C     SUBROUTINE RPAR32(ISTOR,YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE
C             'RPAR'.
C     SUBROUTINE WRIT31(YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE 'WRIT'.
C     SUBROUTINE WRIT32(ISTOR,YL,Y,YY,IY)... SAMPLE SUBROUTINE PACKAGE
C             'WRIT'.
C     SUBROUTINE BLOCK(COOR,ISRF1,ISB1,ISRF2,ISB2,ICB2)... SUBROUTINE
C             PACKAGE 'MODEL'.
C     SUBROUTINE VELOC(IWAVE,UP,US,QP,QS,VP,VS,VD,QL)... SUBROUTINE
C             PACKAGE 'MODEL'.
C     SUBROUTINE SRFC2(ISRF,COOR,F) AND SUBSEQUENT ROUTINES...
C             SUBROUTINE PACKAGE 'SRFC'.
C     SUBROUTINE PARM2(ICB,COOR,UP,US,RO,QP,QS) AND SUBSEQUENT ROUTINES
C             ... SUBROUTINE PACKAGE 'PARM'.
C
C ERROR MESSAGES:
C     581...  TOO MANY FICTITIOUS INTERFACES:
C             MORE THAN 100 FICTITIOUS INTERFACES CROSSED DURING ONE
C             STEP OF THE NUMERICAL INTEGRATION.
C             THIS ERROR SHOULD NOT APPEAR.  CONTACT THE AUTHORS.
C     582,583... EXCLUDED PROGRAM BRANCH:
C             THIS ERROR SHOULD NOT APPEAR.  CONTACT THE AUTHORS.
C     584...  TOO THIN COMPLEX BLOCK:
C             THIS ERROR SHOULD NOT APPEAR.  CONTACT THE AUTHORS.
C     585,586... THIN COMPLEX BLOCK SKIPPED:
C             A THIN COMPLEX BLOCK HAS BEEN SKIPPED OVER WITHIN THE LAST
C             STEP OF THE NUMERICAL INTEGRATION.
C
C DATE: 1990, JANUARY 30
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS FOR LOCAL MODEL PARAMETERS:
      REAL G(12),GAMMA(18),GSQRD,FAUX(10)
      REAL UP(10),US(10),RO,QP,QS,VP,VS,VD(10),QL
C     THESE AUXILIARY VARIABLES AND ARRAYS NEED NOT BE LOCATED IN A
C     COMMON BLOCK.  THERE IS NO REASON TO LOCATE THEM IN THE AUXILIARY
C     COMMON BLOCK /AUXMOD/ BUT TO SHARE THE MEMORY.
      COMMON/AUXMOD/ G,GAMMA,GSQRD,FAUX,UP,US,RO,QP,QS,VP,VS,VD,QL
C
C     G,GAMMA,GSQRD... SEE SUBROUTINE METRIC OF THE PACKAGE 'METRIC'.
C     FAUX... AUXILIARY ARRAY TO STORE A FUNCTIONAL VALUE AND ITS
C             DERIVATIVES.
C     UP,US,RO,QP,QS... SEE SUBROUTINE PARM2 OF THE PACKAGE 'PARM'.
C     UP,US,QP,QS,VP,VS,VD,QL... SEE SUBROUTINE VELOC OF THE PACKAGE
C             'MODEL'.
C
C.......................................................................
C
C     AUXILIARY STORAGE LOCATIONS:
C
      REAL Q111,Q121,Q112,Q122,Q1DET,Q211,Q221,Q212,Q222,Q2DET
      SAVE Q111,Q121,Q112,Q122,Q1DET,Q211,Q221,Q212,Q222,Q2DET
      REAL X1,X2,Y1(MY),Y2(MY),D1(MY),D2(MY)
      SAVE X1,X2,Y1,Y2,D1,D2
      REAL XAUX,YAUX(MY),DAUX(MY),AUX,ERR
      INTEGER IYAUX(12),IYRC12,ISRF2,ISB2,ICB2,K1,K2,K,I
C
C     Q111,Q121,Q112,Q122,Q1DET... MATRIX OF RAY GEOMETRICAL SPREADING
C             AND ITS DETERMINANT AT THE LAST BUT ONE COMPUTED POINT OF
C             THE RAY.
C     Q211,Q221,Q212,Q222,Q2DET... MATRIX OF RAY GEOMETRICAL SPREADING
C             AND ITS DETERMINANT AT THE LAST COMPUTED POINT OF THE RAY.
C     X1,Y1,D1... INDEPENDENT VARIABLE, COMPUTED QUANTITIES AND THEIR
C             DERIVATIVES AT THE LAST BUT ONE COMPUTED POINT OF THE RAY.
C     X2,Y2,D2... INDEPENDENT VARIABLE, COMPUTED QUANTITIES AND THEIR
C             DERIVATIVES AT THE LAST COMPUTED POINT OF THE RAY.
C     IYRC12..VALUE OF IYRC(12)=KMAH AT THE LAST BUT ONE COMPUTED POINT
C             OF THE RAY.
C     XAUX,YAUX,DAUX,AUX,IYAUX,ISRF2,ISB2,ICB2,K1,K2,K,I...  AUXILIARY
C             STORAGE LOCATIONS.
C     ERR...  MAXIMUM ERROR IN INDEPENDENT VARIABLE FOR THE
C             DETERMINATION OF THE POINT OF INTERSECTION.
C
C.......................................................................
C
C     (A) COPYING THE COMPUTED QUANTITIES:
      X1=X2
      X2=X
      DO 11 I=1,NDIM
        Y1(I)=Y2(I)
        Y2(I)=Y(I)
        D1(I)=D2(I)
        D2(I)=D(I)
   11 CONTINUE
      DO 12 I=NDIM+1,IYRC(1)
        Y1(I)=Y(I)
   12 CONTINUE
      IF(X.EQ.PRMT(1)) THEN
C       BEGINNIG OF THE NUMERICAL INTEGRATION
        CALL PSHIFT(27,Y,YI,0.00,0.00,0.00,0.00,0.000,
     *                     Q211,Q221,Q212,Q222,Q2DET,ISHIFT)
        RETURN
      END IF
C
C     (B) NUMBER OF INVOCATIONS OF OUTP
      IYRC(10)=IYRC(10)+1
C     INITIAL VALUE OF THE INDEX OF CROSSED SURFACE BOUNDING THE COMPLEX
C     BLOCK
      IYRC(6)=0
C
C     (C) CHECK FOR CROSSING THE COORDINATE BOUNDARIES OF THE
C     COMPUTATIONAL VOLUME:
      ERR=YYRC(2)*VNEXPS
      DO 31 I=3,5
        IF(Y(I).LT.BOUNDR(I+I-5)) THEN
          IYRC(6)=95+I+I
          FAUX(1)=Y(I)-BOUNDR(I+I-5)
          FAUX(2)=0.
          FAUX(3)=0.
          FAUX(4)=0.
          FAUX(I-1)=1.
          CALL CROSS(IYRC(6),NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,FAUX)
        END IF
        IF(Y(I).GT.BOUNDR(I+I-4)) THEN
          IYRC(6)=96+I+I
          FAUX(1)=Y(I)-BOUNDR(I+I-4)
          FAUX(2)=0.
          FAUX(3)=0.
          FAUX(4)=0.
          FAUX(I-1)=1.
          CALL CROSS(IYRC(6),NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,FAUX)
        END IF
   31 CONTINUE
      IF(Y(1).GT.BOUNDR(7)) THEN
        IYRC(6)=107
        FAUX(1)=Y(I)-BOUNDR(7)
        FAUX(2)=Y(6)*D(I)/D(1)
        FAUX(3)=Y(7)*D(I)/D(1)
        FAUX(4)=Y(8)*D(I)/D(1)
        CALL CROSS(IYRC(6),NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,FAUX)
      END IF
      IF(IYRC(6).GT.100) THEN
        IYRC(7)=IYRC(4)
        IYRC(8)=IABS(IYRC(5))
      END IF
C
C     (D) CHECK FOR CROSSING THE BOUNDARY OF THE COMPLEX BLOCK:
C     NOTE: IYRC(4)=ISB1, IYRC(5)=ICB1, IYRC(6)=ISRF.
      CALL BLOCK(Y(3),0,IYRC(4),ISRF2,ISB2,ICB2,FAUX)
      IF(ISRF2.NE.0) THEN
C       BOUNDARY OF THE SIMPLE BLOCK IS CROSSED
C       NOTE: IN THIS ROUTINE, UNLIKE IN THE PAPER ON C.R.T., THE POINT
C       OF INTERSECTION WITH THE BOUNDARY OF THE SIMPLE BLOCK IS FOUND
C       EVEN IF THE BOUNDARY OF THE COMPLEX BLOCK IS NOT CROSSED.
C       (D1)
        ISBAUX=ISB2
        XAUX=X
        DO 41 I=1,NDIM
          YAUX(I)=Y(I)
          DAUX(I)=D(I)
   41   CONTINUE
C       FOLLOWING LOOP IS INCLUDED TO AVOID INFINITE REPEATING OF THE
C       STEPS (D2) AND (D3) OF THE ALGORITHM
        DO 47 K=1,100
C         (D2)
          IYRC(6)=ISRF2
C         (D3)
          CALL CROSS(IABS(IYRC(6)),NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,
     *                                                             FAUX)
C         X,Y IS THE POINT OF INTERSECTION WITH THE SURFACE IYRC(6)
          CALL BLOCK(Y(3),IYRC(6),IYRC(4),ISRF2,ISB2,ICB2,FAUX)
   43     CONTINUE
          IF(ISRF2.NE.0) THEN
C           (D3-I)
C           X,Y IS NOT SITUATED AT THE BOUNDARY OF THE SIMPLE BLOCK
            IF(FAUX(1)*(Y(6)*FAUX(2)+Y(7)*FAUX(3)+Y(8)*FAUX(4)).LT.0.)
     *                                                              THEN
C             SINCE, IN THE CURRENT VERSION OF THE SUBROUTINE CROSS, THE
C             THE POINT OF INTERSECTION WITH THE SURFACE IYRC(6) IS
C             ALWAYS FOUND AFTER (NOT BEFORE) THE SURFACE, THIS PROGRAM
C             BRANCH SHOULD BE EXCLUDED:
              STOP 'ERROR 582 IN OUTP: EXCLUDED PROGRAM BRANCH'
C             THE POINT OF INTERSECTION WITH THE SURFACE IYRC(6),
C             SEPARATED FROM THE SIMPLE BLOCK IYRC(4) BY THE SURFACE
C             ISRF2, IS SITUATED BEFORE (NOT AFTER) THE SIMPLE BLOCK
C             IYRC(4).  THIS IS LIKELY DUE TO A VERY THIN SIMPLE BLOCK
C             IYRC(4) IN THE VICINITY OF THE POINT OF INTERSECTION.
C             THIS SITUATION IS NOT CONSIDERED IN THE ORIGINAL PAPER
C             ON C.R.T. AND MAY CAUSE SOME DIFFICULTIES:
C             IF(IABS(IYRC(5)).EQ.ICB2) THEN
C               ISRF2=IYRC(6)
C               IYRC(4)=ISB2
C             ELSE
C               STOP 'ERROR 584 IN OUTP: TOO THIN COMPLEX BLOCK'
C             END IF
            END IF
C           THE POINT OF INTERSECTION WITH THE SURFACE IYRC(6) IS
C           FOUND WITHIN THE SPECIFIED ERROR ERR.  IT MAY BE SEPARATED
C           FROM THE SIMPLE BLOCK IYRC(4) BY A SURFACE ISRF2 SITUATED
C           WITHIN THE LAST INTERVAL OF THE LENGTH ERR.  IN SUCH A
C           CASE, THE SURFACE ISRF2 NEED NOT BE TAKEN INTO ACCOUNT.
C           THIS SITUATION IS NOT CONSIDERED IN THE ORIGINAL PAPER
C           ON C.R.T. AND IS EXAMINED IN THE FOLLOWING LINES:
            K2=ISRF2
            DO 44 I=3,5
              Y(I)=Y(I)-D(I)*ERR
   44       CONTINUE
            CALL BLOCK(Y(3),IYRC(6),IYRC(4),ISRF2,ISB2,ICB2,FAUX)
            DO 45 I=3,5
              Y(I)=Y(I)+D(I)*ERR
   45       CONTINUE
            IF(ISRF2.EQ.0) THEN
C             SINCE, IN THE CURRENT VERSION OF THE SUBROUTINE CROSS, THE
C             THE POINT OF INTERSECTION WITH THE SURFACE IYRC(6) IS
C             ALWAYS FOUND AFTER (NOT BEFORE) THE SURFACE, THIS PROGRAM
C             BRANCH SHOULD BE EXCLUDED:
              STOP 'ERROR 583 IN OUTP: EXCLUDED PROGRAM BRANCH'
C             THE FORMERLY FOUND SURFACE ISRF2 WAS SITUATED WITHIN THE
C             LAST INTERVAL OF THE LENGTH ERR.  THUS IT NEED NOT BE
C             TAKEN INTO ACCOUNT:
C             GO TO 43
            ELSE IF(ISRF2.NE.K2) THEN
              IF(FAUX(1)*(Y(6)*FAUX(2)+Y(7)*FAUX(3)+Y(8)*FAUX(4)).LT.0.)
     *                                                              THEN
C               THE POINT OF INTERSECTION WITH THE SURFACE ISRF2 IS
C               IS SITUATED BEFORE (NOT AFTER) THE SIMPLE BLOCK
C               IYRC(4) AND WILL NOT BE CONSIDERED
                ISRF2=K2
              END IF
            END IF
C           GO TO (D2)
          ELSE IF(ICB2.EQ.IABS(IYRC(5))) THEN
C           (D3-II)
C           X,Y IS SITUATED AT THE BOUNDARY OF THE SIMPLE BLOCK
C           BUT NOT SITUATED AT THE BOUNDARY OF THE COMPLEX BLOCK
            IYRC(4)=ISB2
            X=XAUX
            DO 46 I=1,NDIM
              Y(I)=YAUX(I)
              D(I)=DAUX(I)
   46       CONTINUE
            IF(ISB2.EQ.ISBAUX) THEN
C             BOUNDARY OF THE COMPLEX BLOCK HAS NOT BEEN CROSSED
C             DURING THE LAST STEP OF NUMERICAL INTEGRATION
              IYRC(6)=0
              GO TO 49
            END IF
            CALL BLOCK(YAUX(3),0,IYRC(4),ISRF2,ISB2,ICB2,FAUX)
C           GO TO (D2)
          ELSE
C           (D3-III)
C           X,Y IS SITUATED AT THE BOUNDARY OF THE SIMPLE BLOCK AND
C           X,Y IS SITUATED AT THE BOUNDARY OF THE COMPLEX BLOCK
            GO TO 48
          END IF
   47   CONTINUE
        STOP 'ERROR 581 IN OUTP: TOO MANY FICTITIOUS INTERFACES'
   48   CONTINUE
        IYRC(7)=ISB2
        IYRC(8)=ICB2
      END IF
   49 CONTINUE
C
C     (E) CHECK FOR CROSSING THE END SURFACES
      DO 51 I=1,NEND
        IF(KEND(I).GT.NSRFC()) THEN
          CALL SRFC2(KEND(I),Y(3),FAUX)
          IF(FAUX(1)*FSRFCA(KEND(I)-NSRFC()).LE.0.)  THEN
            IYRC(6)=KEND(I)
            CALL CROSS(IYRC(6),NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,FAUX)
          END IF
        END IF
   51 CONTINUE
C
C     (F) PHASE SHIFT DUE TO CAUSTICS
      Q111=Q211
      Q121=Q221
      Q112=Q212
      Q122=Q222
      Q1DET=Q2DET
      CALL PSHIFT(IYRC(1),Y,YI,Q111,Q121,Q112,Q122,Q1DET,
     *                    Q211,Q221,Q212,Q222,Q2DET,I)
      IYRC12=IYRC(12)
      IYRC(12)=IYRC(12)+I
C
C     QUANTITIIES DESCRIBING LOCAL PROPERTIES OF THE MODEL AT POINT X,Y:
      IF(IYRC(6).NE.0) THEN
        CALL PARM2(IABS(IYRC(5)),Y(3),UP,US,YLRC(3),QP,QS)
        CALL VELOC(IYRC(5),UP,US,QP,QS,YLRC(1),YLRC(2),VD,QL)
        YLRC(4)=VD(2)
        YLRC(5)=VD(3)
        YLRC(6)=VD(4)
      END IF
C
C     (G) STORAGE OF THE COMPUTED QUANTITIES AT GIVEN SURFACES:
      DO 70 I=1,3
        IYAUX(I)=IYRC(I)
        IYAUX(I+5)=0.0
        IYAUX(I+8)=IYRC(I+8)
   70 CONTINUE
      IYAUX(5)=IYRC(5)
      DO 72 K=1,NSTOR
        IF(KSTOR(K).GT.NSRFC()) THEN
          IF(KSTOR(K).EQ.IYRC(6)) THEN
            YYRC(1)=X
            CALL RPAR32(K,YLRC,Y,YYRC,IYRC)
            CALL WRIT32(K,YLRC,Y,YYRC,IYRC)
          ELSE
            CALL SRFC2(KSTOR(K),Y(3),FAUX)
            IF(FAUX(1)*FSRFCA(KSTOR(K)-NSRFC()).LE.0.)  THEN
              FSRFCA(KSTOR(K)-NSRFC())=FAUX(1)
              CALL CROSS(KSTOR(K),NDIM,ERR,X1,Y1,D1,X2,Y2,D2,
     *                                              XAUX,YAUX,DAUX,FAUX)
              DO 71 I=NDIM+1,IYRC(1)
                YAUX(I)=Y1(I)
   71         CONTINUE
              CALL PSHIFT(IYRC(1),YAUX,YI,Q111,Q121,Q112,Q122,Q1DET,
     *                        DAUX(1),DAUX(2),DAUX(3),DAUX(4),AUX,I)
              IYAUX(12)=IYRC12+I
              CALL BLOCK(YAUX(3),0,IYRC(4),ISRF2,IYAUX(4),ICB2,FAUX)
              IF(ICB2.NE.IABS(IYRC(5))) THEN
                STOP 'ERROR 585 IN OUTP: THIN COMPLEX BLOCK SKIPPED'
              END IF
              CALL PARM2(IABS(IYRC(5)),YAUX(3),UP,US,DAUX(3),QP,QS)
              CALL VELOC(IYRC(5),UP,US,QP,QS,DAUX(1),DAUX(2),VD,QL)
              DAUX(4)=VD(2)
              DAUX(5)=VD(3)
              DAUX(6)=VD(4)
              YYRC(1)=XAUX
              CALL RPAR32(K,DAUX,YAUX,YYRC,IYAUX)
              CALL WRIT32(K,DAUX,YAUX,YYRC,IYAUX)
            END IF
          END IF
        END IF
   72 CONTINUE
C
C     (H) STORAGE OF THE COMPUTED QUANTITIES ALONG THE RAY:
      IF(STORE.GT.0.) THEN
        K1=INT(X1/STORE+0.000001)+1
        K2=INT(X/STORE+0.000001)
        DO 82 K=K1,K2
          XAUX=FLOAT(K)*STORE
          CALL HIVD2(NDIM,X1,Y1,D1,X2,Y2,D2,XAUX,YAUX,DAUX)
          DO 81 I=NDIM+1,IYRC(1)
            YAUX(I)=Y1(I)
   81     CONTINUE
          CALL PSHIFT(IYRC(1),YAUX,YI,Q111,Q121,Q112,Q122,Q1DET,
     *                    DAUX(1),DAUX(2),DAUX(3),DAUX(4),AUX,I)
          IYAUX(12)=IYRC12+I
          CALL BLOCK(YAUX(3),0,IYRC(4),ISRF2,IYAUX(4),ICB2,FAUX)
          IF(ICB2.NE.IABS(IYRC(5))) THEN
            STOP 'ERROR 586 IN OUTP: THIN COMPLEX BLOCK SKIPPED'
          END IF
          CALL PARM2(IABS(IYRC(5)),YAUX(3),UP,US,DAUX(3),QP,QS)
          CALL VELOC(IYRC(5),UP,US,QP,QS,DAUX(1),DAUX(2),VD,QL)
          DAUX(4)=VD(2)
          DAUX(5)=VD(3)
          DAUX(6)=VD(4)
          YYRC(1)=XAUX
          CALL RPAR31(DAUX,YAUX,YYRC,IYAUX)
          CALL WRIT31(DAUX,YAUX,YYRC,IYAUX)
   82   CONTINUE
      END IF
C
C     (I) ACCUMULATION OF THE RENORMALIZATION ERRORS FOR A TEST OF
C     ACCURACY:
      AUX=0.5*(Y(1)-Y1(1))
      YYRC(3)=YYRC(3)+DELT33*AUX
      YYRC(4)=YYRC(4)+DELT13*AUX
      YYRC(5)=YYRC(5)+DELT11*AUX
C
C     (J) GREAT NUMBER OF BISECTIONS OF THE INITIAL INCREMENT:
      IF(IHLF.GT.NHLF) PRMT(5)=-1.
C
C     (K) FINAL OPERATIONS:
      IF(IYRC(6).NE.0) PRMT(5)=1.
      YYRC(1)=X
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SMVPRD(G,A1,A2,A3,B1,B2,B3)
      REAL G(6),A1,A2,A3,B1,B2,B3
C
C THIS SUBROUTINE IS AN AUXILIARY ROUTINE TO FCT. IT EVALUATES SYMMETRIC
C MATRIX BY VECTOR PRODUCT.
C
C INPUT:
C   G...      ARRAY CONTAINING COMPONENTS G11, G12, G22, G13, G23, G33
C             OF THE 3*3 SYMMETRIC MATRIX.
C   A1,A2,A3... COMPONENTS OF THE 3-VECTOR.
C
C OUTPUT:
C   B1,B2,B3... COMPONENTS OF VECTOR B=G*A.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1989, SEPTEMBER 5
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      B1=G(1)*A1+G(2)*A2+G(4)*A3
      B2=G(2)*A1+G(3)*A2+G(5)*A3
      B3=G(4)*A1+G(5)*A2+G(6)*A3
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE PSHIFT(NY,Y,YI,Q111,Q121,Q112,Q122,Q1DET,
     *                          Q211,Q221,Q212,Q222,Q2DET,ISHIFT)
      INTEGER NY,ISHIFT
      REAL Y(NY),YI(21)
      REAL Q111,Q121,Q112,Q122,Q1DET,Q211,Q221,Q212,Q222,Q2DET
C
C THIS SUBROUTINE IS AN AUXILIARY ROUTINE TO OUTP. IT CORRECTS REDUCED
C AMPLITUDES WITH REGARD TO PHASE SHIFT DUE TO CAUSTICS
C (SEE C.R.T.5.8.3F).
C
C INPUT:
C     NY...   NUMBER OF BASIC QUANTITIES DESCRIBING THE POINT OF A RAY.
C     Y(1:11)... ANYTHING.
C     Y(12:27)... RAY PROPAGATOR MATRIX AT THE POINT 2 OF THE RAY.
C     Y(28:NY)... REDUCED AMPLITUDES AT THE POINT 1 OF THE RAY.
C     YI...   ARRAY CONTAINING QUANTITIES AT THE INITIAL POINT OF THE
C             RAY (SEE C.R.T.6.1)
C     Q111,Q121,Q112,Q122,Q1DET... MATRIX OF RAY GEOMETRICAL SPREADING
C             AND ITS DETERMINANT AT THE POINT 1 OF THE RAY.
C NONE OF THE INPUT PARAMETERS EXCEPT Y(28:NY) ARE ALTERED.
C
C OUTPUT:
C     Y(28:NY)... REDUCED AMPLITUDES AT THE POINT 2 OF THE RAY.
C     Q211,Q221,Q212,Q222,Q2DET... MATRIX OF RAY GEOMETRICAL SPREADING
C             AND ITS DETERMINANT AT THE POINT 2 OF THE RAY.
C     ISHIFT..ORDER OF A CAUSTIC BETWEEN POINTS 1 AND 2 (INCREMENT OF
C             THE KMAH INDEX).
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1989, SEPTEMBER 6
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATION:
      REAL AUX
C
      Q211=Y(12)*YI(12)+Y(16)*YI(13)+Y(20)*YI(14)+Y(24)*YI(15)
      Q221=Y(13)*YI(12)+Y(17)*YI(13)+Y(21)*YI(14)+Y(25)*YI(15)
      Q212=Y(12)*YI(16)+Y(16)*YI(17)+Y(20)*YI(18)+Y(24)*YI(19)
      Q222=Y(13)*YI(16)+Y(17)*YI(17)+Y(21)*YI(18)+Y(25)*YI(19)
      Q2DET=Q211*Q222-Q212*Q221
      IF(Q1DET*Q2DET.LT.0.) THEN
C       CAUSTIC OF THE FIRST ORDER (LINE CAUSTIC)
        ISHIFT=1
        DO 10 I=28,NY,2
          AUX=Y(I+1)
          Y(I+1)=-Y(I)
          Y(I)=AUX
   10   CONTINUE
      ELSE IF((Q111*Q222-Q112*Q221+Q122*Q211-Q121*Q212)*Q2DET.LT.0.)
     *     THEN
C       CAUSTIC OF THE SECOND ORDER (POINT CAUSTIC)
        ISHIFT=2
        DO 20 I=28,NY
          Y(I)=-Y(I)
   20   CONTINUE
      ELSE
C       NO CAUSTIC
        ISHIFT=0
      END IF
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE CROSS(ISRF,NDIM,ERR,X1,Y1,D1,X2,Y2,D2,X,Y,D,F)
      INTEGER ISRF,NDIM
      REAL ERR,X1,Y1(NDIM),D1(NDIM),X2,Y2(NDIM),D2(NDIM)
      REAL X,Y(NDIM),D(NDIM),F(10)
C
C THIS SUBROUTINE FINDS THE POINT OF INTERSECTION OF A CURVE WITH
C A SURFACE (SEE C.R.T.5.8.4B).  THE CURVE IS PARAMETRIZED BY AN
C INDEPENDENT VARIABLE X AND EVALUATED BY THE HERMITE INTERPOLATION FROM
C THE TWO GIVEN POINTS.  THE SURFACE IS SPECIFIED IN AN IMPLICIT WAY BY
C SUBROUTINE SRFC2 WHICH IS DESCRIBED ELSWHERE.
C
C INPUT:
C     ISRF... INDEX OF THE SURFACE. FOR ISRF = 101 TO 107, THE SURFACE
C             COINCIDES WITH A COORDINATE SURFACE BOUNDING A VOLUME FOR
C             COMPUTATION.
C     NDIM... DIMENSION OF ARRAYS Y1,D1,Y2,D2,Y,D.
C     ERR...  MAXIMUM ERROR IN INDEPENDENT VARIABLE FOR THE
C             DETERMINATION OF THE POINT OF INTERSECTION.
C     X1...   INDEPENDENT VARIABLE CORRESPONDING TO THE FIRST POINT
C             GIVEN FOR THE INTERPOLATION OF THE CURVE.
C     Y1...   ARRAY CONTAINING DEPENDENT VARIABLES AT POINT X1.
C             Y1(3), Y1(4), Y1(5) MUST CONTAIN THE COORDINATES OF POINT
C             X1, Y1(6), Y1(7), Y1(8) MUST CONTAIN THE SLOWNESS VECTOR.
C     D1...   ARRAY CONTAINING THE DERIVATIVES OF THE DEPENDENT
C             VARIABLES AT POINT X1.
C     X2...   INDEPENDENT VARIABLE CORRESPONDING TO THE SECOND POINT
C             GIVEN FOR THE INTERPOLATION OF THE CURVE.
C     Y2...   ARRAY CONTAINING DEPENDENT VARIABLES AT POINT X2.
C             Y2(3), Y2(4), Y2(5) MUST CONTAIN THE COORDINATES OF POINT
C             X2, Y2(6), Y2(7), Y2(8) MUST CONTAIN THE SLOWNESS VECTOR.
C     D2...   ARRAY CONTAINING THE DERIVATIVES OF THE DEPENDENT
C             VARIABLES AT POINT X2.
C     X...    INDEPENDENT VARIABLE CORRESPONDING TO THE POINT OF THE
C             CURVE AT WHICH THE FUNCTION SPECIFYING THE SURFACE HAS THE
C             OPPOSITE SIGN THAN AT X1. THEN THE POINT OF INTERSECTION
C             IS BEING FOUND BETWEEN THE POINTS X1 AND X.  THE FOUND
C             APPROXIMATION OF THE POINT OF INTERSECTION IS SITUATED
C             CLOSE TO THE SURFACE, AT THE OPOSITE SIDE THAN THE GIVEN
C             POINT X1.
C     Y...    ARRAY OF THE DIMENSION AT LEAST NDIM.
C     D...    ARRAY CONTAINING THE DERIVATIVES OF THE DEPENDENT
C             VARIABLES AT POINT X.  D(3), D(4), D(5) MUST CONTAIN THE
C             DERIVATIVES OF COORDINATES WITH RESPECT TO X, AT THE POINT
C             X.  OTHER STORAGE LOCATIONS MAY BE UNDEFINED.
C     F...    ARRAY CONTAINING THE VALUE, FIRST AND SECOND DERIVATIVES
C             OF THE FUNCTION SPECIFYING THE SURFACE AT POINT X.
C ISRF, NDIM, ERR, X1, Y1, D1, X2, Y2, D2 ARE UNALTERED.
C
C OUTPUT:
C     X...    INDEPENDENT VARIABLE CORRESPONDING TO THE POINT OF
C             INTERSECTION.
C     Y...    ARRAY CONTAINING DEPENDENT VARIABLES AT THE POINT OF
C             INTERSECTION.
C     D...    ARRAY CONTAINING THE DERIVATIVES OF THE DEPENDENT
C             VARIABLES AT THE POINT OF INTERSECTION.
C     F...    UNDEFINED.
C
C COMMON BLOCK /DCRT/ (SEE SUBROUTINE PACKAGE 'RAY'):
      INTEGER MEND,MSTOR
      PARAMETER (MEND=128)
      PARAMETER (MSTOR=128)
      INTEGER KSTORE,NEXPS,NHLF,MODCRT
      REAL STORE,STEP,UEB,UEBPP,UEBPH,UEBHH,UEBDRT,BOUNDR(7)
      INTEGER NSRFCA,NEND,KEND(MEND),NSTOR,KSTOR(MSTOR)
      COMMON/DCRT/ KSTORE,NEXPS,NHLF,MODCRT,STORE,STEP,UEB,UEBPP,UEBPH,
     *             UEBHH,UEBDRT,BOUNDR,NSRFCA,NEND,KEND,NSTOR,KSTOR
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL SRFC2,HIVD2
C     SUBROUTINE SRFC2(ISRF,COOR,F) AND SUBSEQUENT ROUTINES...
C             SUBROUTINE PACKAGE 'SRFC'.
C     SUBROUTINE HIVD2(NDIM,X1,Y1,D1,X2,Y2,D2,X,Y,D)... THIS SUBROUTINE
C             PACKAGE.
C
C ERROR MESSAGES:
C     588...  NO POINT OF INTERSECTION:
C             THE FUNCTION SPECIFYING THE SURFACE HAS THE SAME SIGN AT
C             THE GIVEN POINTS X1 AND X.  THUS, THE BOTH POINTS ARE
C             SITUATED AT THE SAME SIDE OF THE SURFACE AND THERE IS NO
C             GUARANTY TO FIND A POINT OF INTERSECTION BETWEEN THEM.
C     589...  TOO MANY ITERATIONS:
C             MORE THAN 20 ITERATIONS WHEN DETERMINING THE POINT OF
C             INTERSECTION OF THE RAY WITH A SURFACE.
C             THIS ERROR SHOULD NOT APPEAR.  CONTACT THE AUTHORS.
C
C DATE: 1990, JANUARY 15
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
      INTEGER I,ITER
      REAL XA,FA,DA,XB,FB,DB,XC,XCA,XCB,FOLD,RATE
C
C
C     INITIAL VALUES:
      XA=X
      FA=F(1)
      DA=F(2)*D(3)+F(3)*D(4)+F(4)*D(5)
      XB=X1
      X =X1
      DO 1 I=1,NDIM
        Y(I)=Y1(I)
        D(I)=D1(I)
    1 CONTINUE
      FOLD=FA
C     RATE OF THE GIVEN MAXIMUM ERROR AND A DESIRED ERROR
      RATE=5.
C
C     ITERATIONS:
      DO 2 ITER=1,20
C
C       FUNCTIONAL VALUE AT X:
        IF(ISRF.LE.100) THEN
          CALL SRFC2(ISRF,Y(3),F)
        ELSE IF(ISRF.LE.106) THEN
          I=(ISRF-97)/2
          F(1)=Y(I+1)-BOUNDR(ISRF-100)
          F(2)=0.
          F(3)=0.
          F(4)=0.
          F(I)=1.
        ELSE
          I=ISRF-106
          F(1)=Y(I)-BOUNDR(ISRF-100)
          F(2)=Y(6)*D(I)/D(1)
          F(3)=Y(7)*D(I)/D(1)
          F(4)=Y(8)*D(I)/D(1)
        END IF
C
C       SELECTION OF POINTS:
        IF(FA*F(1).GT.0.) THEN
          IF(ITER.EQ.1) THEN
            STOP 'ERROR 588 IN CROSS: NO POINT OF INTERSECTION'
          END IF
          XA=XB
          FA=FB
          DA=DB
        END IF
C
C       NEW POINT OR END OF ITERATIONS:
        IF(ABS(X-XA).LE.ERR) THEN
C         POINT OF INTERSECTION IS FOUND WITHIN THE SPECIFIED ERROR ERR
          IF(F(1)*FOLD.GT.0.) THEN
C           POINT X IS SITUATED AT THE OTHER SIDE OF THE SURFACE THAN X1
            GO TO 3
          ELSE IF(ABS(X-XA)*RATE.LE.ERR) THEN
C           POINT X IS SITUATED AT THE SAME SIDE OF THE SURFACE AS X1
            X=XA
            GO TO 3
          ELSE
C           ALLOWING FOR ONE MORE ITERATION
            RATE=0.
          END IF
        END IF
        XB=X
        FB=F(1)
        DB=F(2)*D(3)+F(3)*D(4)+F(4)*D(5)
C
C       NEW APPROXIMATION:
        IF(MOD(ITER,2).EQ.1) THEN
C         REGULA FALSI:
          X=(FA*XB-FB*XA)/(FA-FB)
        ELSE
C         MODIFIED NEWTON-RAPHSON:
          XCA=XA-FA/DA
          XCB=XB-FB/DB
          XC=(XA+XB)/2.
          IF((XCA-XC)*(XCA-XB).LT.0.) THEN
            IF((XCB-XC)*(XCB-XB).LT.0.) THEN
              IF(ABS(XCA-XB).LT.ABS(XCB-XB)) THEN
                X=XCA
              ELSE
                X=XCB
              END IF
            ELSE
              X=XCA
            END IF
          ELSE
            IF((XCB-XC)*(XCB-XB).LT.0.) THEN
              X=XCB
            ELSE
              X=XC
            END IF
          END IF
        END IF
C
C       MODIFICATION FOR ROUNDING ERRORS:
        X=X+SIGN(ERR/50.,XA-XB)
C
C       INTERPOLATION OF THE RAY:
        CALL HIVD2(3,X1,Y1(3),D1(3),X2,Y2(3),D2(3),X,Y(3),D(3))

    2 CONTINUE
C     END OF LOOP FOR ITERATIONS
      STOP 'ERROR 589 IN CROSS: TOO MANY ITERATIONS'
C
    3 CONTINUE
      CALL HIVD2(NDIM,X1,Y1,D1,X2,Y2,D2,X,Y,D)
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE HIVD2(NDIM,X1,Y1,D1,X2,Y2,D2,X,Y,D)
      INTEGER NDIM
      REAL X1,Y1(NDIM),D1(NDIM),X2,Y2(NDIM),D2(NDIM)
      REAL X,Y(NDIM),D(NDIM)
C
C THIS SUBROUTINE PERFORMS HERMITE INTERPOLATION OF A VECTOR AND ITS
C DERIVATIVES USING FUNCTIONAL VALUES AND DERIVATIVES AT 2 GIVEN POINTS.
C
C INPUT:
C     NDIM... DIMENSION OF ARRAYS Y1,D1,Y2,D2,Y,D (THE NUMBER OF
C             INDEPENDENT VARIABLES).
C     X1...   INDEPENDENT VARIABLE CORRESPONDING TO THE FIRST GIVEN
C             POINT.
C     Y1...   ARRAY CONTAINING FUNCTIONAL VALUES AT THE FIRST GIVEN
C             POINT.
C     D1...   ARRAY CONTAINING THE DERIVATIVES AT THE FIRST GIVEN POINT.
C     X2...   INDEPENDENT VARIABLE CORRESPONDING TO THE SECOND GIVEN
C             POINT.
C     Y2...   ARRAY CONTAINING FUNCTIONAL VALUES AT THE SECOND GIVEN
C             POINT.
C     D2...   ARRAY CONTAINING THE DERIVATIVES AT THE SECOND GIVEN POINT
C     X...    INDEPENDENT VARIABLE OF THE POINT AT WHICH THE
C             INTERPOLATED VECTOR IS TO BE EVALUATED.
C NONE OF THE INPUT PARAMETERS ARE ALTERED.
C
C OUTPUT:
C     Y...    ARRAY CONTAINING INTERPOLATED FUNCTIONAL VALUES AT X.
C     D...    ARRAY CONTAINING THE DERIVATIVES OF THE INTERPOLATED
C             FUNCTIONAL VALUES AT X.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1989, OCTOBER 20
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
      INTEGER I
      REAL A,B,A1,A2,B1,B2,DA1,DB1,DB2
C
C     SUBSTITUTIONS:
      A=(X-X2)/(X1-X2)
      B=(A-1.)*A
C
C     BASIC FUNCTIONS:
      A1=(A-B-B)*A
      A2=1.-A1
      B1=B*(X-X2)
      B2=B*(X-X1)
C     DERIVATIVES OF BASIC FUNCTIONS:
      DA1=6.*B/(X2-X1)
      DB1=3.*B+A
      DB2=3.*B+1.-A
C
C     INTERPOLATION:
      DO 1 I=1,NDIM
        Y(I)=A1*Y1(I)+A2*Y2(I)+B1*D1(I)+B2*D2(I)
        D(I)=DA1*(Y1(I)-Y2(I))+DB1*D1(I)+DB2*D2(I)
    1 CONTINUE
C
      RETURN
      END
C
C=======================================================================
C
