C SUBROUTINE PACKAGE 'SCRONUL' - DUMMY SCREEN OUTPUT SUBROUTINES
C
C BY VLASTISLAV CERVENY, LUDEK KLIMES, IVAN PSENCIK
C
C THIS PACKAGE CONSISTS OF THE FOLLOWING EXTERNAL PROCEDURES:
C     SCRO1...SCREEN OUTPUT SUBROUTINE CALLED WHEN STARTING THE COMPLETE
C             RAY TRACING PROGRAM, AND WHEN STARTING THE COMPUTATION OF
C             A NEW ELEMENTARY WAVE.
C     SCRO2...SCREEN OUTPUT SUBROUTINE CALLED WHEN STARTING THE COMPLETE
C             TRACING OF A NEW RAY.
C     SCRO3...SCREEN OUTPUT SUBROUTINE CALLED WITH CONSTANT STEP STORE
C             OF THE INDEPENDENT VARIABLE ALONG THE RAY, AND AT THE
C             POINTS OF INTERSECTION WITH INTERFACES EITHER BEFORE AND
C             AFTER THE TRANSFORMATION.
C     SCRO4...SCREEN OUTPUT SUBROUTINE CALLED AFTER TERMINATION OF
C             TRACING THE RAY.
C     SCRO5...SCREEN OUTPUT SUBROUTINE CALLED AFTER TERMINATION OF THE
C             COMPUTATION OF AN ELEMENTARY WAVE, AND WHEN TERMINATING
C             THE COMPLETE RAY TRACING PROGRAM.
C
C=======================================================================
C
      SUBROUTINE SCRO1(IWAVE)
      INTEGER IWAVE
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED WHEN STARTING THE COMPLETE RAY
C TRACING PROGRAM, AND WHEN STARTING THE COMPUTATION OF A NEW ELEMENTARY
C WAVE.
C
C INPUT:
C     IWAVE...ZERO WHEN STARTING THE COMPLETE RAY TRACING PROGRAM,
C             OTHERWISE THE INDEX OF THE ELEMENTARY WAVE WHICH WILL BE
C             COMPUTED (I.E. THE OUTPUT OF THE SUBROUTINE CODE1 FROM THE
C             PACKAGE 'CODE').
C
C NO OUTPUT.
C
C DATE: 1990, JANUARY 22
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO2(IRAY)
      INTEGER IRAY
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED WHEN STARTING THE COMPLETE
C TRACING OF A NEW RAY.
C
C INPUT:
C     IRAY... THE INDEX OF THE RAY WHICH WILL BE COMPUTED (I.E. THE
C             OUTPUT OF THE SUBROUTINE RPAR2 FROM THE PACKAGE 'RPAR').
C
C NO OUTPUT.
C
C DATE: 1989, DECEMBER 4
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO3(YL,Y,YY,IY)
      REAL YL(6),Y(35),YY(5)
      INTEGER IY(12)
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED WITH CONSTANT STEP STORE OF
C THE INDEPENDENT VARIABLE ALONG THE RAY, AND AT THE POINTS OF
C INTERSECTION WITH INTERFACES EITHER BEFORE AND AFTER THE
C TRANSFORMATION.  IT PLOTS THE PART OF THE RAY COMPUTED IN THE LAST
C STEP OF THE NUMERICAL INTEGRATION.  IT IS CALLED BY THE SUBROUTINE
C WRIT31.
C
C INPUT:
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.
C NONE OF THE INPUT PARAMETERS ARE ALTERED.
C
C NO OUTPUT.
C
C DATE: 1989, DECEMBER 4
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO4(IRAY,YL,Y,YY,IY,IEND)
C
      INTEGER IRAY,IY(12),IEND
      REAL YL(6),Y(35),YY(5)
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED AFTER TERMINATION OF TRACING
C THE RAY.
C
C INPUT:
C     IRAY... THE INDEX OF THE RAY WHICH HAS BEEN COMPUTED (I.E. THE
C             OUTPUT OF THE SUBROUTINE RPAR2 FROM THE PACKAGE 'RPAR').
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.
C     IEND... REASON OF THE TERMINATION OF THE COMPUTATION OF A RAY (SEE
C             C.R.T.5.4).  FOR A DETAILED DESCRIPTION SEE SUBROUTINE RAY
C             (SUBROUTINE PACKAGE 'RAY').
C
C NO OUTPUT.
C
C DATE: 1989, DECEMBER 4
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO5(IWAVE)
      INTEGER IWAVE
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED AFTER TERMINATION OF THE
C COMPUTATION OF AN ELEMENTARY WAVE, AND WHEN TERMINATING THE COMPLETE
C RAY TRACING PROGRAM.
C
C INPUT:
C     IWAVE...ZERO WHEN TERMINATING THE COMPLETE RAY TRACING PROGRAM,
C             OTHERWISE THE INDEX OF THE ELEMENTARY WAVE WHICH HAS BEEN
C             COMPUTED (I.E. THE OUTPUT OF THE SUBROUTINE CODE1 FROM THE
C             PACKAGE 'CODE').
C
C NO OUTPUT.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
C     SUBROUTINE PLOT(XPAGE,YPAGE,IPEN)... CALCOMP GRAPHICS SUBROUTINE.
C
C DATE: 1990, JANUARY 22
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
      RETURN
      END
C
C=======================================================================
C
