C SUBROUTINE PACKAGE 'SCROPC' - SCREEN OUTPUT SUBROUTINES FOR PC'S
C
C BY VLASTISLAV CERVENY, LUDEK KLIMES, IVAN PSENCIK
C
C THIS PACKAGE CONSISTS OF:
C     SCROB...BLOCK DATA SUBROUTINE DEFINING AUXILIARY COMMON BLOCK
C             /SCROC/ TO CONFIGURE THE SCREEN OUTPUT.
C     SCRO1...SCREEN OUTPUT SUBROUTINE CALLED WHEN STARTING THE COMPLETE
C             RAY TRACING PROGRAM, AND WHEN STARTING THE COMPUTATION OF
C             A NEW ELEMENTARY WAVE.
C     SCRO2...SCREEN OUTPUT SUBROUTINE CALLED WHEN STARTING THE COMPLETE
C             TRACING OF A NEW RAY.
C     SCRO3...SCREEN OUTPUT SUBROUTINE CALLED WITH CONSTANT STEP STORE
C             OF THE INDEPENDENT VARIABLE ALONG THE RAY, AND AT THE
C             POINTS OF INTERSECTION WITH INTERFACES EITHER BEFORE AND
C             AFTER THE TRANSFORMATION.
C     SCRO4...SCREEN OUTPUT SUBROUTINE CALLED AFTER TERMINATION OF
C             TRACING THE RAY.
C     SCRO5...SCREEN OUTPUT SUBROUTINE CALLED AFTER TERMINATION OF THE
C             COMPUTATION OF AN ELEMENTARY WAVE, AND WHEN TERMINATING
C             THE COMPLETE RAY TRACING PROGRAM.
C     CURSOR..CHARACTER FUNCTION THAT RETURNS THE ANSI ESCAPE SEQUENCE
C             POSITIONING THE CURSOR AT THE BEGINNING OF THE GIVEN LINE.
C     SPECIFICATION OF THE USED CALCOMP GRAPHICS SUBROUTINES.
C
C CONFIGURATION OF THE SCREEN OUTPUT:
C     THE QUANTITIES SPECIFYING THE SCALE AND OTHER PROPERTIES OF THE
C     SIMPLE GRAPHIC OUTPUT ARE STORED IN THE COMMON BLOCK /SCROC/
C     DEFINED IN THE FOLLOWING SUBROUTINE:
C     ------------------------------------------------------------------
      BLOCK DATA SCROB
      INTEGER MKOLOR
      PARAMETER (MKOLOR=16)
      INTEGER NKOLOR,KOLOR(MKOLOR)
      REAL HMIN,HMAX,VMIN,VMAX,WIDTH
      REAL H1A,H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
      COMMON/SCROC/NKOLOR,KOLOR,HMIN,HMAX,VMIN,VMAX,WIDTH,H1A,
     *             H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
      SAVE /SCROC/
C     DIMENSIONS OF THE PLOT AREA:
      DATA HMIN,HMAX,VMIN,VMAX,WIDTH/7.45,29.66,0.00,21.00,0.045/
C     COLOURS:
      DATA NKOLOR/6/,KOLOR/2,3,5,6,4,7,14,1,1,1,1,1,1,1,1,1/
      END
C     ------------------------------------------------------------------
C     MKOLOR..DIMENSION OF THE ARRAY KOLOR.
C     NKOLOR..NUMBER OF DIFFERENT COLOURS USED TO PLOT DIFFERENT
C             ELEMENTS OF A RAY.
C     KOLOR...ARRAY CONTAINING THE INDICES OF COLOURS USED TO PLOT RAYS.
C             KOLOR(I)... THE COLOR OF EACH I-TH ELEMENT OF A RAY.  IF
C               THE NUMBER OF RAY ELEMENTS IS GREATER THAN NKOLOR, THE
C               COLORS KOLOR(1) TO KOLOR(NKOLOR) ARE CYCLICLY REPEATED.
C             KOLOR(NKOLOR+1)... INDEX OF THE COLOUR TO PLOT THE FRAMES
C               AROUND THE THREE PROJECTIONS OF THE MODEL BOX.
C     HMIN, HMAX... HORIZONTAL COORDINATES OF THE VERTICAL BOUNDARIES OF
C             THE PLOTTING AREA IN THE PLOT COORDINATES.
C     VMIN, VMAX... VERTICAL COORDINATES OF THE HORIZONTAL BOUNDARIES OF
C             THE PLOTTING AREA IN THE PLOT COORDINATES.
C     WIDTH...THICKNESS OF THE PLOTTED LINE.
C     H1A,H1B,H2A,H2B,V2A,V2B,V3A,V3B... AUXILIARY VARIABLES:
C             COEFFICIENTS OF THE LINEAR FUNCTIONS PROJECTING GENERAL
C             COORDINATES OF A POINT ON A RAY TO THE PLOT COORDINATES.
C     H1OLD,H2OLD,V2OLD,V3OLD... AUXILIARY VARIABLES: POSITIONS OF THE
C             PROJECTIONS OF THE LAST POINT OF A RAY TO THE SCREEN IN
C             PLOT COORDINATES.
C
C DATE: 1990, JANUARY 22
C CODED BY LUDEK KLIMES
C
C=======================================================================
C
      SUBROUTINE SCRO1(IWAVE)
      INTEGER IWAVE
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED WHEN STARTING THE COMPLETE RAY
C TRACING PROGRAM, AND WHEN STARTING THE COMPUTATION OF A NEW ELEMENTARY
C WAVE.
C
C INPUT:
C     IWAVE...ZERO WHEN STARTING THE COMPLETE RAY TRACING PROGRAM,
C             OTHERWISE THE INDEX OF THE ELEMENTARY WAVE WHICH WILL BE
C             COMPUTED (I.E. THE OUTPUT OF THE SUBROUTINE CODE1 FROM THE
C             PACKAGE 'CODE').
C
C NO OUTPUT.
C
C COMMON BLOCK /DCRT/ (SEE SUBROUTINE PACKAGE 'RAY'):
      INTEGER MEND,MSTOR
      PARAMETER (MEND=128)
      PARAMETER (MSTOR=128)
      INTEGER KSTORE,NEXPS,NHLF,MODCRT
      REAL STORE,STEP,UEB,UEBPP,UEBPH,UEBHH,UEBDRT,BOUNDR(7)
      INTEGER NSRFCA,NEND,KEND(MEND),NSTOR,KSTOR(MSTOR)
      COMMON/DCRT/ KSTORE,NEXPS,NHLF,MODCRT,STORE,STEP,UEB,UEBPP,UEBPH,
     *             UEBHH,UEBDRT,BOUNDR,NSRFCA,NEND,KEND,NSTOR,KSTOR
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C COMMON BLOCK /SCROC/:
      INTEGER MKOLOR
      PARAMETER (MKOLOR=16)
      INTEGER NKOLOR,KOLOR(MKOLOR)
      REAL HMIN,HMAX,VMIN,VMAX,WIDTH
      REAL H1A,H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
      COMMON/SCROC/NKOLOR,KOLOR,HMIN,HMAX,VMIN,VMAX,WIDTH,H1A,
     *             H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
C STORAGE LOCATIONS H1B,H2A,H2B,V2A,V2B,V3A,V3B OF THE COMMON BLOCK ARE
C DEFINED IN THIS SUBROUTINE.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL CURSOR,PLOTS,PLOT,NEWPEN
      CHARACTER*8 CURSOR
C     FUNCTION CURSOR(LINE)... THIS PACKAGE.
C     SUBROUTINE PLOTS(0,0,0)... CALCOMP GRAPHICS SUBROUTINE.
C     SUBROUTINE PLOT(XPAGE,YPAGE,IPEN)... CALCOMP GRAPHICS SUBROUTINE.
C     SUBROUTINE NEWPEN(INP)... CALCOMP GRAPHICS SUBROUTINE.
C
C ERROR MESSAGES:
C     560...  TOO MANY COLOURS:
C             THE DIMENSION MKOLOR OF THE ARRAY KOLOR MUST BE GREATER
C             THAN THE NUMBER NKOLOR OF COLOURS USED TO PLOT THE RAY
C             ELEMENTS, SEE THE DATA BLOCK SUBROUTINE SCROB.
C
C DATE: 1990, FEBRUARY 5
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
      REAL STREET,H1,H2,V1,V2
C
      IF(IWAVE.GT.0) THEN
C
C       ERASING SCREEN:
        WRITE(*,'(A)') CURSOR(0)
C
C       DIMENSIONS OF THE PLOT AREA:
        STREET=2.*WIDTH
C
C       COEFFICIENTS OF THE LINEAR PROJECTIONS ONTO THE SCREEN:
        H1A=((HMAX-HMIN-STREET)/2.-3.*WIDTH)/(BOUNDR(2)-BOUNDR(1))
        H1B=HMIN+1.5*WIDTH-BOUNDR(1)*H1A
        H2A=((HMAX-HMIN-STREET)/2.-3.*WIDTH)/(BOUNDR(4)-BOUNDR(3))
        H2B=HMAX-1.5*WIDTH-BOUNDR(4)*H2A
        V2A=((VMAX-VMIN-STREET)/2.-3.*WIDTH)/(BOUNDR(4)-BOUNDR(3))
        V2B=VMAX-1.5*WIDTH-BOUNDR(4)*V2A
        V3A=((VMAX-VMIN-STREET)/2.-3.*WIDTH)/(BOUNDR(6)-BOUNDR(5))
        V3B=VMIN+1.5*WIDTH-BOUNDR(5)*V3A
C
C       PLOT INITIALIZATION:
        CALL PLOTS(0,0,0)
        IF(NKOLOR.GE.MKOLOR) THEN
          STOP 'ERROR 560 IN SCRO1: TOO MANY COLOURS'
        END IF
        CALL NEWPEN(KOLOR(NKOLOR+1))
C
C       PLOTTING FRAMES:
        H1=HMIN+WIDTH/2.
        H2=(HMIN+HMAX-STREET-WIDTH)/2.
        V1=(VMIN+VMAX+STREET+WIDTH)/2.
        V2=VMAX-WIDTH/2.
        CALL PLOT(H1,V1,3)
        CALL PLOT(H2,V1,2)
        CALL PLOT(H2,V2,2)
        CALL PLOT(H1,V2,2)
        CALL PLOT(H1,V1,2)
        V1=VMIN+WIDTH/2.
        V2=(VMIN+VMAX-STREET-WIDTH)/2.
        CALL PLOT(H1,V1,3)
        CALL PLOT(H2,V1,2)
        CALL PLOT(H2,V2,2)
        CALL PLOT(H1,V2,2)
        CALL PLOT(H1,V1,2)
        H1=(HMIN+HMAX+STREET+WIDTH)/2.
        H2=HMAX-WIDTH/2.
        CALL PLOT(H1,V1,3)
        CALL PLOT(H2,V1,2)
        CALL PLOT(H2,V2,2)
        CALL PLOT(H1,V2,2)
        CALL PLOT(H1,V1,2)
C
C       WRITING TO THE SCREEN:
        WRITE(*,'(A,A,I8)') CURSOR(3),'WAVE:',IWAVE
        WRITE(*,'(A,A)')    CURSOR(2),'                    '
        WRITE(*,'(A,A)')    CURSOR(1),'NEW ELEMENTARY WAVE '
C
      END IF
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO2(IRAY)
      INTEGER IRAY
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED WHEN STARTING THE COMPLETE
C TRACING OF A NEW RAY.
C
C INPUT:
C     IRAY... THE INDEX OF THE RAY WHICH WILL BE COMPUTED (I.E. THE
C             OUTPUT OF THE SUBROUTINE RPAR2 FROM THE PACKAGE 'RPAR').
C
C NO OUTPUT.
C
C COMMON BLOCK /INITC/ (SEE SUBROUTINE PACKAGE 'INIT'):
      INTEGER MSRFCA
      PARAMETER (MSRFCA=128)
      INTEGER ISB1I,ICB1I
      REAL YLI(6),YI(25),FSRFCA(MSRFCA)
      COMMON/INITC/ISB1I,ICB1I,YLI,YI,FSRFCA
C NONE OF THE STORAGE LOCATIONS OF THE COMMON BLOCK ARE ALTERED.
C
C COMMON BLOCK /SCROC/:
      INTEGER MKOLOR
      PARAMETER (MKOLOR=16)
      INTEGER NKOLOR,KOLOR(MKOLOR)
      REAL HMIN,HMAX,VMIN,VMAX,WIDTH
      REAL H1A,H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
      COMMON/SCROC/NKOLOR,KOLOR,HMIN,HMAX,VMIN,VMAX,WIDTH,H1A,
     *             H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
C STORAGE LOCATIONS H1OLD,H2OLD,V2OLD,V3OLD OF THE COMMON BLOCK ARE
C DEFINED IN THIS SUBROUTINE.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL CURSOR
      CHARACTER*8 CURSOR
C     FUNCTION CURSOR(LINE)... THIS PACKAGE.
C
C DATE: 1990, FEBRUARY 5
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     NO AUXILIARY STORAGE LOCATIONS.
C
C     WRITING TO THE SCREEN:
      WRITE(*,'(A,A)')       CURSOR(1),'COMPUTING           '
      WRITE(*,'(A,A,I8)')    CURSOR(4),'RAY: ',IRAY
      WRITE(*,'(A,A,F15.6)') CURSOR(5),'PAR1:',YI(20)
      WRITE(*,'(A,A,F15.6)') CURSOR(6),'PAR2:',YI(21)
      WRITE(*,'(A,A)')       CURSOR(7),'                    '
      WRITE(*,'(A,A)')       CURSOR(8),'        ISB ICB ISRF'
C
C     INITIAL POSITION FOR PLOTTING THE RAY:
      H1OLD=YI(3)*H1A+H1B
      H2OLD=YI(4)*H2A+H2B
      V2OLD=YI(4)*V2A+V2B
      V3OLD=YI(5)*V3A+V3B
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO3(YL,Y,YY,IY)
      REAL YL(6),Y(35),YY(5)
      INTEGER IY(12)
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED WITH CONSTANT STEP STORE OF
C THE INDEPENDENT VARIABLE ALONG THE RAY, AND AT THE POINTS OF
C INTERSECTION WITH INTERFACES EITHER BEFORE AND AFTER THE
C TRANSFORMATION.  IT PLOTS THE PART OF THE RAY COMPUTED IN THE LAST
C STEP OF THE NUMERICAL INTEGRATION.  IT IS CALLED BY THE SUBROUTINE
C WRIT31.
C
C INPUT:
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.
C NONE OF THE INPUT PARAMETERS ARE ALTERED.
C
C NO OUTPUT.
C
C COMMON BLOCK /SCROC/:
      INTEGER MKOLOR
      PARAMETER (MKOLOR=16)
      INTEGER NKOLOR,KOLOR(MKOLOR)
      REAL HMIN,HMAX,VMIN,VMAX,WIDTH
      REAL H1A,H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
      COMMON/SCROC/NKOLOR,KOLOR,HMIN,HMAX,VMIN,VMAX,WIDTH,H1A,
     *             H1B,H2A,H2B,V2A,V2B,V3A,V3B,H1OLD,H2OLD,V2OLD,V3OLD
C STORAGE LOCATIONS H1OLD,H2OLD,V2OLD,V3OLD OF THE COMMON BLOCK ARE
C REDEFINED IN THIS SUBROUTINE.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL NELEM,PLOT,NEWPEN
      INTEGER NELEM
C     FUNCTION NELEM(KODIND)... SUBROUTINE PACKAGE 'CODE'.
C     SUBROUTINE PLOT(XPAGE,YPAGE,IPEN)... CALCOMP GRAPHICS SUBROUTINE.
C     SUBROUTINE NEWPEN(INP)... CALCOMP GRAPHICS SUBROUTINE.
C
C DATE: 1990, JANUARY 18
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
      REAL H1NEW,H2NEW,V2NEW,V3NEW
C
C     WRITING TO THE SCREEN:
      IF(IY(6).NE.0) THEN
        IF(IY(8).NE.0) THEN
          WRITE(*,'('' .......'',3I4)') IY(4),IY(5),IY(6)
        ELSE
          WRITE(*,'(''        '',3I4)') IY(4),IY(5),IY(6)
        END IF
      END IF
C
C     PLOTTING THE RAY AT THE SCREEN:
      H1NEW=Y(3)*H1A+H1B
      H2NEW=Y(4)*H2A+H2B
      V2NEW=Y(4)*V2A+V2B
      V3NEW=Y(5)*V3A+V3B
      CALL NEWPEN(KOLOR(MOD(NELEM(IY(2))-1,NKOLOR)+1))
      CALL PLOT(H1OLD,V2OLD,3)
      CALL PLOT(H1NEW,V2NEW,2)
      CALL PLOT(H1OLD,V3OLD,3)
      CALL PLOT(H1NEW,V3NEW,2)
      CALL PLOT(H2OLD,V3OLD,3)
      CALL PLOT(H2NEW,V3NEW,2)
      H1OLD=H1NEW
      H2OLD=H2NEW
      V2OLD=V2NEW
      V3OLD=V3NEW
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO4(IRAY,YL,Y,YY,IY,IEND)
C
      INTEGER IRAY,IY(12),IEND
      REAL YL(6),Y(35),YY(5)
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED AFTER TERMINATION OF TRACING
C THE RAY.
C
C INPUT:
C     IRAY... THE INDEX OF THE RAY WHICH HAS BEEN COMPUTED (I.E. THE
C             OUTPUT OF THE SUBROUTINE RPAR2 FROM THE PACKAGE 'RPAR').
C     YL...   ARRAY CONTAINING LOCAL QUANTITIES AT THE POINT OF THE RAY.
C     Y...    ARRAY CONTAINING BASIC QUANTITIES COMPUTED ALONG THE RAY.
C     YY...   ARRAY CONTAINING REAL AUXILIARY QUANTITIES COMPUTED ALONG
C             THE RAY.
C     IY...   ARRAY CONTAINING INTEGER AUXILIARY QUANTITIES COMPUTED
C             ALONG THE RAY.
C     IEND... REASON OF THE TERMINATION OF THE COMPUTATION OF A RAY (SEE
C             C.R.T.5.4).  FOR A DETAILED DESCRIPTION SEE SUBROUTINE RAY
C             (SUBROUTINE PACKAGE 'RAY').
C
C NO OUTPUT.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL CURSOR,NELEM
      CHARACTER*8 CURSOR
      INTEGER NELEM
C     FUNCTION CURSOR(LINE)... THIS PACKAGE.
C     NELEM(KODIND)... SUBROUTINE PACKAGE 'CODE'.
C
C DATE: 1990, FEBRUARY 5
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     AUXILIARY STORAGE LOCATIONS:
      INTEGER I,NLINES
C
C     NUMBER OF LINES AVAILABLE ON THE SCREEN: NLINES:
C     NLINES MAY BE LOWER THAN THE ACTUAL NUMBER OF LINES BUT SHOULD NOT
C     EXCEED THAT.  NLINES=25 IS A GOOD CHOICE FOR IBM-PC'S, BUT
C     NLINES=24 FITS ALSO VAX COMPUTERS.
      PARAMETER (NLINES=24)
C
C     WRITING TO THE SCREEN:
      WRITE(*,'('' END:'',I3,3I4)') IEND,IY(4),IY(5),IY(6)
      DO 10 I=1,NLINES-9-2*NELEM(IY(2))
        WRITE(*,'(''                     '')')
   10 CONTINUE
      WRITE(*,'(A,A)') CURSOR(1),'                    '
C
      RETURN
      END
C
C=======================================================================
C
      SUBROUTINE SCRO5(IWAVE)
      INTEGER IWAVE
C
C THIS SCREEN OUTPUT SUBROUTINE IS CALLED AFTER TERMINATION OF THE
C COMPUTATION OF AN ELEMENTARY WAVE, AND WHEN TERMINATING THE COMPLETE
C RAY TRACING PROGRAM.
C
C INPUT:
C     IWAVE...ZERO WHEN TERMINATING THE COMPLETE RAY TRACING PROGRAM,
C             OTHERWISE THE INDEX OF THE ELEMENTARY WAVE WHICH HAS BEEN
C             COMPUTED (I.E. THE OUTPUT OF THE SUBROUTINE CODE1 FROM THE
C             PACKAGE 'CODE').
C
C NO OUTPUT.
C
C SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED:
      EXTERNAL CURSOR,PLOT
      CHARACTER*8 CURSOR
C     FUNCTION CURSOR(LINE)... THIS PACKAGE.
C     SUBROUTINE PLOT(XPAGE,YPAGE,IPEN)... CALCOMP GRAPHICS SUBROUTINE.
C
C DATE: 1990, JANUARY 22
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     NO AUXILIARY STORAGE LOCATIONS.
C
      IF(IWAVE.EQ.0) THEN
C       ERASING TEXT SCREEN:
        WRITE(*,'(A)') CURSOR(0)
      ELSE
C       CLOSING DOWN PLOTTING:
        CALL PLOT(0.0,0.0,999)
      END IF
      RETURN
      END
C
C=======================================================================
C
      CHARACTER*8 FUNCTION CURSOR(LINE)
      INTEGER LINE
C
C THIS FUNCTION RETURNS THE ANSI ESCAPE SEQUENCE POSITIONING THE CURSOR
C AT THE BEGINNING OF THE GIVEN LINE.
C
C INPUT:
C     LINE... INDEX OF THE GIVEN LINE, OR ZERO.  IF LINE=0, THE ENTIRE
C             SCREEN IS TO BE ERASED AND THE CURSOR RETURNED TO THE HOME
C             POSITION.
C
C OUTPUT:
C     CERSOR..ESCAPE SEQUENCE POSITIONING THE CURSOR AT THE BEGINNING OF
C             THE GIVEN LINE, OR ERASING THE SCREEN.
C
C NO SUBROUTINES AND EXTERNAL FUNCTIONS REQUIRED.
C
C DATE: 1990, JANUARY 18
C CODED BY LUDEK KLIMES
C
C-----------------------------------------------------------------------
C
C     NO AUXILIARY STORAGE LOCATIONS.
C
      IF(LINE.EQ.0) THEN
        CURSOR='    '//CHAR(27)//CHAR(91)//'2J'
      ELSE
        CURSOR=' '//CHAR(27)//CHAR(91)//
     *         CHAR(48+LINE/10)//CHAR(48+MOD(LINE,10))//CHAR(59)//'1H'
      END IF
      RETURN
      END
C
C=======================================================================
C
C SPECIFICATION OF THE USED CALCOMP GRAPHICS SUBROUTINES:
C     SUBROUTINE PLOTS(I1,I2,I3)... INITIALIZES PLOTTING.  IT IS CALLED
C             WHEN STARTING THE COMPUTATION OF THE NEW ELEMENTARY WAVE.
C             I1,I2,I3... SET TO ZEROS.
C     SUBROUTINE PLOT(XPAGE,YPAGE,IPEN)... MOVES PEN OR PLOTS A LINE.
C             XPAGE, YPAGE... COORDINATES, OFTEN IN CENTIMETRES.
C             IPEN... CONTROLS THE PLOTTING:
C               IPEN=2... THE PEN IS DOWN DURING THE MOVEMENT, THUS
C                 DRAWING A LINE.
C               IPEN=3... THE PEN IS UP DURING THE MOVEMENT.
C               IPEN=999... TERMINATES PLOTTING INITIALIZED BY THE
C                 SUBROUTINE PLOTS.
C     SUBROUTINE NEWPEN(INP)... CHANGES THE COLOR.
C             INP... SPECIFIES THE NUMBER OF COLOR TO BE SELECTED.
C     ALL PARAMETERS ARE INPUT PARAMETERS AND SHOULD NOT BE MODIFIED.
C
C=======================================================================
C
